<?php

// FeedsHTTPFetcher plugin is needed, as this class will use almost all of the same methods, etc.
// It just needs to override enough functions to get to the FeedsHTTPBatch::getRaw() method, which
// is where the error logging and URL-fixing needs to take place.

module_load_include('inc', 'feeds', 'plugins/FeedsHTTPFetcher');

class FeedsLogHTTPBatch extends FeedsHTTPBatch{
	
    /**
     * Override of FeedsHTTPBatch::getRaw().
     * This is done to implement logging of bad status codes and fixing of redirected URLs.
     */
     public function getRaw() {
     	//feeds_include_library('http_request.inc', 'http_request');
     	watchdog('feeds_id', $this->id);
     	// ** use overridden http_request file to report 301 status codes from CURL **
     	module_load_include('inc', 'feeds_log', 'http_request_override');
     	
     	$result = http_request_get($this->url);
     	
     	// Change this variable in order to update the URL of the feed when it encounters a 302.
     	// Since 302s are technically "temporary," the safer, more conservative solution is to NOT
     	// update the URLs; however, it may be beneficial to automatically update URLs that return
     	// 302, especially if curl is disabled/unavailable, since in that case, feeds will not 
     	// automatically follow the redirect and retrieve content.
     	
     	$update_302_urls = FALSE;
     	
     	if (!in_array($result->code, array(200, 201, 202, 203, 204, 205, 206))) {
     		
     		// CHANGES: if 301, update the feed url.
     		// Otherwise, log the error before throwing an exception.
     		
     		if ($result->code == 301 || ($result->code == 302 && $update_302_urls)){
  				
  				$curl = FALSE;
  				if (http_request_use_curl()){
  					$curl = TRUE;
  					// Redirected URL is already present in response information
  					$redirected = $result->headers['Location'];
  				}
  				else{
  					// Get redirected URL without use of CURL functions
     				$redirected = $this->get_final_url();
     			}
     			
     			// Update the URL in the FeedsSource object and save it to the database
  
     			$src = feeds_source('feed', $this->feed_nid);
     			$config = $src->getConfig();
     			$config[key($config)]['source'] = $redirected;
     			$src->setConfig($config);
     			$src->save();
     			
     			$messageStr = t('URL of feed !nid redirected to @redirected -- feed updated', array('!nid' => $this->feed_nid, '@redirected' => $redirected));
     			watchdog('feeds_log', $messageStr);
     			
     			if (!$curl){
     				throw new Exception($messageStr);
     			}
     			
     		}
     		else{
     			$existing = db_fetch_object(db_query("SELECT url, fail_count FROM {feeds_log} WHERE feed_nid = %d", $this->feed_nid));
     			if (!$existing){
     				db_query("INSERT INTO {feeds_log} (feed_nid, url, status_code, fail_count, last_checked)
     					VALUES (%d, '%s', %d, %d, %d)", $this->feed_nid, $this->url, $result->code, 1, date('U'));
     			}
     			else{
     				// If the URL has not changed since the last log entry, increment the fail count.
     				if ($this->url == $existing->url){
     					db_query("UPDATE {feeds_log} SET fail_count = %d, last_checked = %d WHERE feed_nid = %d", ($existing->fail_count + 1), date('U'), $this->feed_nid);
     				}
     				// If the URL has changed since last log entry, this should count as a new log entry
     				// with new information and a fail count of 1.
     				else{
     					db_query("UPDATE {feeds_log} SET url = '%s', status_code = %d, fail_count = %d, last_checked = %d WHERE feed_nid = %d",
     						$this->url, $result->code, 1, date('U'), $this->feed_nid);
     				}
     			}
     			
     			// We also want to unpublish nodes that produce bad status codes. We can't do this
     			// via node_load and node_save, however, because feeds_nodeapi width $op == 'update'
     			// expects $node to come from a form, which will set 'feeds'. Instead, just modify
     			// the status field directly in the database.
     			
     			db_query("UPDATE {node} SET status = %d WHERE nid = %d", 0, $this->feed_nid);
   				
   				// If a 302 error brought us here and CURL is enabled, we don't need to throw an exception,
   				// since CURL enables us to automatically follow the redirect and still retrieve
   				// content. Only throw an exception if the URL can't be followed and there is 
   				// therefore no data:
   				if (!($result->code == 302 && $curl)){
     				throw new Exception(t('Download of @url failed with code !code.', array('@url' => $this->url, '!code' => $result->code)));
     			}
     		}
     		
     	}
     	return $result->data;
     }
     
     
     // These 3 methods were written by the blogger at w-shadow.com.
     // This technique was chosen over using CURL, because not all servers have CURL enabled
     // and some users may not have the power to enable it.
     // To see the original post where this code is presented, visit:
     // http://w-shadow.com/blog/2008/07/05/how-to-get-redirect-url-in-php/
     
     /**
      * Gets the address that the provided URL points to, or FALSE if there's no redirect.
      * This code came from a blog post at w-shadow.com.
      * See post: http://w-shadow.com/blog/2008/07/05/how-to-get-redirect-url-in-php/
      *
      * @param url
      * @return string
      */
     public function get_redirect_url($url){
     	$redirect_url = null; 
		$url_parts = @parse_url($url);
		if (!$url_parts) return false;
		if (!isset($url_parts['host'])) return false; //can't process relative URLs
		if (!isset($url_parts['path'])) $url_parts['path'] = '/';
	 
		$sock = fsockopen($url_parts['host'], (isset($url_parts['port']) ? (int)$url_parts['port'] : 80), $errno, $errstr, 30);
		if (!$sock) return false;
	 
		$request = "HEAD " . $url_parts['path'] . (isset($url_parts['query']) ? '?'.$url_parts['query'] : '') . " HTTP/1.1\r\n"; 
		$request .= 'Host: ' . $url_parts['host'] . "\r\n"; 
		$request .= "Connection: Close\r\n\r\n"; 
		fwrite($sock, $request);
		$response = '';
		while(!feof($sock)) $response .= fread($sock, 8192);
		fclose($sock);
	 
		if (preg_match('/^Location: (.+?)$/m', $response, $matches)){
			if ( substr($matches[1], 0, 1) == "/" )
				return $url_parts['scheme'] . "://" . $url_parts['host'] . trim($matches[1]);
			else
				return trim($matches[1]);
	 
		} else {
			return false;
		}
     }
     
     /**
      * Follows and collects all redirects, in order, for the given URL.
      * This code came from a blog post at w-shadow.com.
      * See post: http://w-shadow.com/blog/2008/07/05/how-to-get-redirect-url-in-php/
      *
      * @param url
      * @return array
      */
     public function get_all_redirects($url){
     	$redirects = array();
		while ($newurl = $this->get_redirect_url($url)){
			if (in_array($newurl, $redirects)){
				break;
			}
			$redirects[] = $newurl;
			$url = $newurl;
		}
		return $redirects;

     }
     
     /**
      * Gets the final address that the URL leads to.
      * This code came from a blog post at w-shadow.com.
      * See post: http://w-shadow.com/blog/2008/07/05/how-to-get-redirect-url-in-php/
      *
      * Returns the original url if it isn't really a redirect.
      *
      */
     public function get_final_url(){
     	$redirects = $this->get_all_redirects($this->url);
		if (count($redirects) > 0){
			return array_pop($redirects);
		} else {
			return $url;
		}
     }
    
}

class FeedsLogHTTPFetcher extends FeedsHTTPFetcher{
	
	/**
	 * Override of FeedsHTTPFetcher::fetch().
	 *
	 * This allows us to use our new class (FeedsHTTPLogBatch) in place of FeedsHTTPBatch.
	 */
	public function fetch(FeedsSource $source) {
		$config = $source->getConfig();
		if (isset($config['FeedsLogHTTPFetcher'])) $source_config = $config['FeedsLogHTTPFetcher'];
		else if (isset($config['FeedsHTTPFetcher'])) $source_config = $config['FeedsHTTPFetcher'];
		else $source_config = $config[key($config)];
		
		if ($this->config['use_pubsubhubbub'] && ($raw = $this->subscriber($source->feed_nid)->receive())) {
			return new FeedsImportBatch($raw, $source->feed_nid);
		}
		return new FeedsLogHTTPBatch($source_config['source'], $source->feed_nid);
	}
}