<?php

/**
 * Implementation of hook_views_handlers().
 *
 * Registers a new handler to provide a human-readable translation of the HTTP status code that
 * caused the error.
 */
function feeds_log_views_handlers(){
	return array(
		'handlers' => array(
			'feeds_log_views_handler_status_code_text' => array(
				'parent' => 'views_handler_field',
				'file' => 'feeds_log_views_handler_status_code_text.inc',
			),
		),
	);
}

/**
 * Implementation of hook_views_data().
 *
 * Exposes feeds_log table to Views for creating report of broken links in Feeds.
 */
function feeds_log_views_data(){
	$data['feeds_log'] = array(
		'table' => array(
			'group' => t('Feeds Error Log'),
			'join' => array(
				'node' => array(
					'left_table' => 'node',
					'left_field' => 'nid',
					'field' => 'feed_nid',
				),
			),
		),
		// Feed URL
		'url' => array(
			'title' => t('URL'),
			'help' => t('The URL of a feed with a failed HTTP request'),
			'field' => array(
				'handler' => 'views_handler_field',
				'click sortable' => TRUE,
			),
			'sort' => array(
				'handler' => 'views_handler_sort',
			),
			'filter' => array(
				'handler' => 'views_handler_filter_string',
			),
			'argument' => array(
				'handler' => 'views_handler_argument_string',
			),
		),
		// HTTP Response Code
		'status_code' => array(
			'title' => t('Status Code'),
			'help' => t('The status code returned by HTTP requests to the feed URL'),
			'field' => array(
				'handler' => 'views_handler_field_numeric',
				'click sortable' => TRUE,
			),
			'sort' => array(
				'handler' => 'views_handler_sort',
			),
			'filter' => array(
				'handler' => 'views_handler_filter_numeric',
			),
		),
		// Human-readable translation of status code
		'status_code_text' => array(
			'field' => array(
				'title' => t('Status Code Meaning'),
				'help' => t('A human-readable translation of the HTTP status code'),
				'handler' => 'feeds_log_views_handler_status_code_text',
			),
		),
		// Fail Count
		'fail_count' => array(
			'title' => t('Fail Count'),
			'help' => t('The number of times requests for the URL have failed'),
			'field' => array(
				'handler' => 'views_handler_field_numeric',
				'click sortable' => TRUE,
			),
			'sort' => array(
				'handler' => 'views_handler_sort',
			),
			'filter' => array(
				'handler' => 'views_handler_filter_numeric',
			),
		),
		// Last Checked
		'last_checked' => array(
			'title' => t('Last Checked'),
			'help' => t('The time importing was attempted from the URL'),
			'field' => array(
				'handler' => 'views_handler_field_date',
				'click sortable' => TRUE,
			),
			'sort' => array(
				'handler' => 'views_handler_sort_date',
			),
			'filter' => array(
				'handler' => 'views_handler_filter_date',
			),
		),
	);
	return $data;
}

/**
 * Implementation of hook_views_default_views().
 *
 * Used to provide a default feeds log report that can be overriden if desired.
 */
function feeds_log_views_default_views(){
	$views = array();
	
	$view = new view;
	$view->name = 'feeds_error_log';
	$view->description = 'Displays log entries for feeds that failed on import.';
	$view->tag = '';
	$view->view_php = '';
	$view->base_table = 'node';
	$view->is_cacheable = FALSE;
	$view->api_version = 2;
	$view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
	$handler = $view->new_display('default', 'Defaults', 'default');
	$handler->override_option('fields', array(
	  'edit_node' => array(
		'label' => '',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'text' => '[edit]',
		'exclude' => 0,
		'id' => 'edit_node',
		'table' => 'node',
		'field' => 'edit_node',
		'relationship' => 'none',
	  ),
	  'url' => array(
		'label' => 'URL',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 1,
		  'path' => '[url]',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 1,
		  'max_length' => '40',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'exclude' => 0,
		'id' => 'url',
		'table' => 'feeds_log',
		'field' => 'url',
		'relationship' => 'none',
	  ),
	  'title' => array(
		'label' => 'Title',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'link_to_node' => 1,
		'exclude' => 0,
		'id' => 'title',
		'table' => 'node',
		'field' => 'title',
		'relationship' => 'none',
	  ),
	  'status_code' => array(
		'label' => 'Status Code',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'set_precision' => FALSE,
		'precision' => 0,
		'decimal' => '.',
		'separator' => ',',
		'prefix' => '',
		'suffix' => '',
		'exclude' => 0,
		'id' => 'status_code',
		'table' => 'feeds_log',
		'field' => 'status_code',
		'relationship' => 'none',
	  ),
	  'status_code_text' => array(
		'label' => 'Meaning',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 1,
		'empty_zero' => 0,
		'exclude' => 0,
		'id' => 'status_code_text',
		'table' => 'feeds_log',
		'field' => 'status_code_text',
		'relationship' => 'none',
	  ),
	  'fail_count' => array(
		'label' => 'Fail Count',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'set_precision' => FALSE,
		'precision' => 0,
		'decimal' => '.',
		'separator' => ',',
		'prefix' => '',
		'suffix' => '',
		'exclude' => 0,
		'id' => 'fail_count',
		'table' => 'feeds_log',
		'field' => 'fail_count',
		'relationship' => 'none',
	  ),
	  'last_checked' => array(
		'label' => 'Last Checked',
		'alter' => array(
		  'alter_text' => 0,
		  'text' => '',
		  'make_link' => 0,
		  'path' => '',
		  'link_class' => '',
		  'alt' => '',
		  'prefix' => '',
		  'suffix' => '',
		  'target' => '',
		  'help' => '',
		  'trim' => 0,
		  'max_length' => '',
		  'word_boundary' => 1,
		  'ellipsis' => 1,
		  'html' => 0,
		  'strip_tags' => 0,
		),
		'empty' => '',
		'hide_empty' => 0,
		'empty_zero' => 0,
		'date_format' => 'small',
		'custom_date_format' => '',
		'exclude' => 0,
		'id' => 'last_checked',
		'table' => 'feeds_log',
		'field' => 'last_checked',
		'relationship' => 'none',
	  ),
	));
	$handler->override_option('filters', array(
	  'type' => array(
		'operator' => 'in',
		'value' => array(
		  'feed' => 'feed',
		),
		'group' => '0',
		'exposed' => FALSE,
		'expose' => array(
		  'operator' => FALSE,
		  'label' => '',
		),
		'id' => 'type',
		'table' => 'node',
		'field' => 'type',
		'relationship' => 'none',
	  ),
	  'status_code' => array(
		'operator' => '!=',
		'value' => array(
		  'value' => '0',
		  'min' => '',
		  'max' => '',
		),
		'group' => '0',
		'exposed' => FALSE,
		'expose' => array(
		  'operator' => FALSE,
		  'label' => '',
		),
		'id' => 'status_code',
		'table' => 'feeds_log',
		'field' => 'status_code',
		'relationship' => 'none',
	  ),
	));
	$handler->override_option('access', array(
	  'type' => 'none',
	));
	$handler->override_option('cache', array(
	  'type' => 'none',
	));
	$handler->override_option('title', 'Feeds Error Log');
	$handler->override_option('style_plugin', 'table');
	$handler->override_option('style_options', array(
	  'grouping' => '',
	  'override' => 1,
	  'sticky' => 0,
	  'order' => 'desc',
	  'columns' => array(
		'edit_node' => 'edit_node',
		'url' => 'url',
		'title' => 'title',
		'status_code' => 'status_code',
		'status_code_text' => 'status_code_text',
		'fail_count' => 'fail_count',
		'last_checked' => 'last_checked',
	  ),
	  'info' => array(
		'edit_node' => array(
		  'separator' => '',
		),
		'url' => array(
		  'sortable' => 0,
		  'separator' => '',
		),
		'title' => array(
		  'sortable' => 1,
		  'separator' => '',
		),
		'status_code' => array(
		  'sortable' => 1,
		  'separator' => '',
		),
		'status_code_text' => array(
		  'separator' => '',
		),
		'fail_count' => array(
		  'sortable' => 1,
		  'separator' => '',
		),
		'last_checked' => array(
		  'sortable' => 1,
		  'separator' => '',
		),
	  ),
	  'default' => 'last_checked',
	));
	$handler = $view->new_display('page', 'Page', 'page_1');
	$handler->override_option('path', 'feeds_error_log');
	$handler->override_option('menu', array(
	  'type' => 'none',
	  'title' => '',
	  'description' => '',
	  'weight' => 0,
	  'name' => 'navigation',
	));
	$handler->override_option('tab_options', array(
	  'type' => 'none',
	  'title' => '',
	  'description' => '',
	  'weight' => 0,
	  'name' => 'navigation',
	));

	$views[$view->name] = $view;
	return $views;
}